@@ -3,6 +3,7 @@
  This file is part of the KDE project.
 
 Copyright (C) 2014 Martin Gräßlin <mgraesslin@kde.org>
+Copyright (C) 2019 Kevin Ottens <kevin.ottens@enioka.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +26,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../greeter/lnf_integration.h"
 
 #include <config-kscreenlocker.h>
-#include <KActionCollection>
+#include <KConfigLoader>
+#include <KConfigDialogManager>
 #include <KGlobalAccel>
 #include <KCModule>
 #include <KPluginFactory>
@@ -38,9 +40,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QQmlContext>
 #include <QQuickItem>
 
-static const QString s_lockActionName = QStringLiteral("Lock Session");
-static const QString s_defaultWallpaperPackage = QStringLiteral("org.kde.image");
-
 class ScreenLockerKcmForm : public QWidget, public Ui::ScreenLockerKcmForm
 {
     Q_OBJECT
@@ -62,122 +61,65 @@ ScreenLockerKcmForm::ScreenLockerKcmForm(QWidget *parent)
 
 ScreenLockerKcm::ScreenLockerKcm(QWidget *parent, const QVariantList &args)
     : KCModule(parent, args)
-    , m_actionCollection(new KActionCollection(this, QStringLiteral("ksmserver")))
+    , m_settings(new KScreenSaverSettings(this))
     , m_ui(new ScreenLockerKcmForm(this))
 {
     QVBoxLayout* layout = new QVBoxLayout(this);
     layout->addWidget(m_ui);
-
-    addConfig(KScreenSaverSettings::self(), m_ui);
-
-    m_actionCollection->setConfigGlobal(true);
-    QAction *a = m_actionCollection->addAction(s_lockActionName);
-    a->setProperty("isConfigurationAction", true);
-    m_ui->lockscreenShortcut->setCheckForConflictsAgainst(KKeySequenceWidget::None);
-    a->setText(i18n("Lock Session"));
-    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>{Qt::ALT+Qt::CTRL+Qt::Key_L, Qt::Key_ScreenSaver});
-    connect(m_ui->lockscreenShortcut, &KKeySequenceWidget::keySequenceChanged, this, &ScreenLockerKcm::shortcutChanged);
-
-    loadWallpapers();
-    auto wallpaperChangedSignal = static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged);
-    connect(m_ui->wallpaperCombo, wallpaperChangedSignal, this, static_cast<void (KCModule::*)()>(&ScreenLockerKcm::changed));
-    connect(m_ui->wallpaperCombo, wallpaperChangedSignal, this, &ScreenLockerKcm::loadWallpaperConfig);
-
-    m_ui->wallpaperCombo->installEventFilter(this);
+    for (const auto &pluginInfo : m_settings->availableWallpaperPlugins()) {
+        m_ui->kcfg_wallpaperPluginIndex->addItem(pluginInfo.name, pluginInfo.id);
+    }
+    connect(m_ui->kcfg_wallpaperPluginIndex, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+            this, &ScreenLockerKcm::loadWallpaperConfig);
+    m_ui->kcfg_wallpaperPluginIndex->installEventFilter(this);
+    m_ui->installEventFilter(this);
 
     auto proxy = new ScreenLockerProxy(this);
+
     m_ui->wallpaperConfigWidget->setClearColor(m_ui->palette().color(QPalette::Active, QPalette::Window));
     m_ui->wallpaperConfigWidget->rootContext()->setContextProperty(QStringLiteral("configDialog"), proxy);
+    m_ui->wallpaperConfigWidget->setSource(QUrl(QStringLiteral("qrc:/kscreenlocker-kcm-resources/wallpaperconfig.qml")));
+    connect(m_ui->wallpaperConfigWidget->rootObject(), SIGNAL(configurationChanged()), this, SLOT(updateState()));
 
     m_ui->lnfConfigWidget->setClearColor(m_ui->palette().color(QPalette::Active, QPalette::Window));
     m_ui->lnfConfigWidget->rootContext()->setContextProperty(QStringLiteral("configDialog"), proxy);
-
-
-
-    connect(this, &ScreenLockerKcm::wallpaperConfigurationChanged, proxy, &ScreenLockerProxy::wallpaperConfigurationChanged);
-    connect(this, &ScreenLockerKcm::currentWallpaperChanged, proxy, &ScreenLockerProxy::currentWallpaperChanged);
-
-    m_ui->wallpaperConfigWidget->setSource(QUrl(QStringLiteral("qrc:/kscreenlocker-kcm-resources/wallpaperconfig.qml")));
-    connect(m_ui->wallpaperConfigWidget->rootObject(), SIGNAL(configurationChanged()), this, SLOT(changed()));
-
     m_ui->lnfConfigWidget->setSource(QUrl(QStringLiteral("qrc:/kscreenlocker-kcm-resources/lnfconfig.qml")));
-    connect(m_ui->lnfConfigWidget->rootObject(), SIGNAL(configurationChanged()), this, SLOT(changed()));
+    connect(m_ui->lnfConfigWidget->rootObject(), SIGNAL(configurationChanged()), this, SLOT(updateState()));
 
-    m_ui->installEventFilter(this);
-}
-
-void ScreenLockerKcm::shortcutChanged(const QKeySequence &key)
-{
-    if (QAction *a = m_actionCollection->action(s_lockActionName)) {
-        auto shortcuts = KGlobalAccel::self()->shortcut(a);
-        m_ui->lockscreenShortcut->setProperty("changed", !shortcuts.contains(key));
-    }
-    changed();
+    addConfig(m_settings, m_ui);
 }
 
 void ScreenLockerKcm::load()
 {
     KCModule::load();
 
-    m_package = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("Plasma/LookAndFeel"));
-    KConfigGroup cg(KSharedConfig::openConfig(QStringLiteral("kdeglobals")), "KDE");
-    const QString packageName = cg.readEntry("LookAndFeelPackage", QString());
-    if (!packageName.isEmpty()) {
-        m_package.setPath(packageName);
-    }
-
-    if (QAction *a = m_actionCollection->action(s_lockActionName)) {
-        auto shortcuts = KGlobalAccel::self()->shortcut(a);
-        if (!shortcuts.isEmpty()) {
-            m_ui->lockscreenShortcut->setKeySequence(shortcuts.first());
-        }
-    }
-    m_lnfIntegration = new ScreenLocker::LnFIntegration(this);
-    m_lnfIntegration->setPackage(m_package);
-    m_lnfIntegration->setConfig(KScreenSaverSettings::self()->sharedConfig());
-    m_lnfIntegration->init();
-
-
-    selectWallpaper(KScreenSaverSettings::self()->wallpaperPlugin());
     loadWallpaperConfig();
     loadLnfConfig();
-}
 
-void ScreenLockerKcm::test(const QString &plugin)
-{
-    if (plugin.isEmpty() || plugin == QLatin1String("none")) {
-        return;
+    if (m_lnfSettings) {
+        m_lnfSettings->load();
+        emit m_lnfSettings->configChanged(); // To force the ConfigPropertyMap to reevaluate
     }
 
-    QProcess proc;
-    QStringList arguments;
-    arguments << plugin << QStringLiteral("--testing");
-    if (proc.execute(QString::fromLatin1(KSCREENLOCKER_GREET_BIN), arguments)) {
-        QMessageBox::critical(this, i18n("Error"), i18n("Failed to successfully test the screen locker."));
+    if (m_wallpaperSettings) {
+        m_wallpaperSettings->load();
+        emit m_wallpaperSettings->configChanged(); // To force the ConfigPropertyMap to reevaluate
     }
+
+    updateState();
 }
 
 void ScreenLockerKcm::save()
 {
-    if (!shouldSaveShortcut()) {
-        QMetaObject::invokeMethod(this, "changed", Qt::QueuedConnection);
-        return;
-    }
     KCModule::save();
-    QMetaObject::invokeMethod(m_ui->wallpaperConfigWidget->rootObject(), "saveConfig");
-    QMetaObject::invokeMethod(m_ui->lnfConfigWidget->rootObject(), "saveConfig");
-
-    // set the wallpaper config
-    KScreenSaverSettings::self()->setWallpaperPlugin(m_ui->wallpaperCombo->currentData().toString());
-
-    KScreenSaverSettings::self()->save();
-    if (m_ui->lockscreenShortcut->property("changed").toBool()) {
-        if (QAction *a = m_actionCollection->action(s_lockActionName)) {
-            KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>{m_ui->lockscreenShortcut->keySequence()}, KGlobalAccel::NoAutoloading);
-            m_actionCollection->writeSettings();
-        }
-        m_ui->lockscreenShortcut->setProperty("changed", false);
+    if (m_lnfSettings) {
+        m_lnfSettings->save();
     }
+
+    if (m_wallpaperSettings) {
+        m_wallpaperSettings->save();
+    }
+
     // reconfigure through DBus
     OrgKdeScreensaverInterface interface(QStringLiteral("org.kde.screensaver"),
                                          QStringLiteral("/ScreenSaver"),
@@ -185,56 +127,50 @@ void ScreenLockerKcm::save()
     if (interface.isValid()) {
         interface.configure();
     }
-}
 
-bool ScreenLockerKcm::shouldSaveShortcut()
-{
-    if (m_ui->lockscreenShortcut->property("changed").toBool()) {
-        const QKeySequence &sequence = m_ui->lockscreenShortcut->keySequence();
-        auto conflicting = KGlobalAccel::getGlobalShortcutsByKey(sequence);
-        if (!conflicting.isEmpty()) {
-            // Inform and ask the user about the conflict and reassigning
-            // the keys sequence
-            if (!KGlobalAccel::promptStealShortcutSystemwide(this, conflicting, sequence)) {
-                return false;
-            }
-            KGlobalAccel::stealShortcutSystemwide(sequence);
-        }
-    }
-    return true;
+    updateState();
 }
 
 void ScreenLockerKcm::defaults()
 {
     KCModule::defaults();
-    m_ui->lockscreenShortcut->setKeySequence(Qt::ALT+Qt::CTRL+Qt::Key_L);
 
-    selectWallpaper(s_defaultWallpaperPackage);
+    if (m_lnfSettings) {
+        m_lnfSettings->setDefaults();
+        emit m_lnfSettings->configChanged(); // To force the ConfigPropertyMap to reevaluate
+    }
+
+    if (m_wallpaperSettings) {
+        m_wallpaperSettings->setDefaults();
+        emit m_wallpaperSettings->configChanged(); // To force the ConfigPropertyMap to reevaluate
+    }
+
+    updateState();
 }
 
-void ScreenLockerKcm::loadWallpapers()
+void ScreenLockerKcm::updateState()
 {
-    const auto wallpaperPackages = KPackage::PackageLoader::self()->listPackages(QStringLiteral("Plasma/Wallpaper"));
-    for (auto &package : wallpaperPackages) {
-        m_ui->wallpaperCombo->addItem(package.name(), package.pluginId());
-    }
-}
+    bool isDefaults = m_settings->isDefaults();
+    bool isSaveNeeded = m_settings->isSaveNeeded();
 
-void ScreenLockerKcm::selectWallpaper(const QString &pluginId)
-{
-    const auto index = m_ui->wallpaperCombo->findData(pluginId);
-    if (index != -1) {
-        m_ui->wallpaperCombo->setCurrentIndex(index);
-    } else if (pluginId != s_defaultWallpaperPackage) {
-        // fall back to default plugin
-        selectWallpaper(s_defaultWallpaperPackage);
+    if (m_lnfSettings) {
+        isDefaults &= m_lnfSettings->isDefaults();
+        isSaveNeeded |= m_lnfSettings->isSaveNeeded();
     }
+
+    if (m_wallpaperSettings) {
+        isDefaults &= m_wallpaperSettings->isDefaults();
+        isSaveNeeded |= m_wallpaperSettings->isSaveNeeded();
+    }
+
+    emit changed(isSaveNeeded);
+    emit defaulted(isDefaults);
 }
 
 void ScreenLockerKcm::loadWallpaperConfig()
 {
     if (m_wallpaperIntegration) {
-        if (m_wallpaperIntegration->pluginName() == m_ui->wallpaperCombo->currentData().toString()) {
+        if (m_wallpaperIntegration->pluginName() == m_ui->kcfg_wallpaperPluginIndex->currentData().toString()) {
             // nothing changed
             return;
         }
@@ -243,9 +179,10 @@ void ScreenLockerKcm::loadWallpaperConfig()
     emit currentWallpaperChanged();
 
     m_wallpaperIntegration = new ScreenLocker::WallpaperIntegration(this);
-    m_wallpaperIntegration->setConfig(KScreenSaverSettings::self()->sharedConfig());
-    m_wallpaperIntegration->setPluginName(m_ui->wallpaperCombo->currentData().toString());
+    m_wallpaperIntegration->setConfig(m_settings->sharedConfig());
+    m_wallpaperIntegration->setPluginName(m_ui->kcfg_wallpaperPluginIndex->currentData().toString());
     m_wallpaperIntegration->init();
+    m_wallpaperSettings = m_wallpaperIntegration->configScheme();
     m_ui->wallpaperConfigWidget->rootContext()->setContextProperty(QStringLiteral("wallpaper"), m_wallpaperIntegration);
     emit wallpaperConfigurationChanged();
     m_ui->wallpaperConfigWidget->rootObject()->setProperty("sourceFile", m_wallpaperIntegration->package().filePath(QByteArrayLiteral("ui"), QStringLiteral("config.qml")));
@@ -253,6 +190,25 @@ void ScreenLockerKcm::loadWallpaperConfig()
 
 void ScreenLockerKcm::loadLnfConfig()
 {
+    if (m_package.isValid() && m_lnfIntegration) {
+        return;
+    }
+
+    Q_ASSERT(!m_package.isValid() && !m_lnfIntegration);
+
+    m_package = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("Plasma/LookAndFeel"));
+    KConfigGroup cg(KSharedConfig::openConfig(QStringLiteral("kdeglobals")), "KDE");
+    const QString packageName = cg.readEntry("LookAndFeelPackage", QString());
+    if (!packageName.isEmpty()) {
+        m_package.setPath(packageName);
+    }
+
+    m_lnfIntegration = new ScreenLocker::LnFIntegration(this);
+    m_lnfIntegration->setPackage(m_package);
+    m_lnfIntegration->setConfig(m_settings->sharedConfig());
+    m_lnfIntegration->init();
+    m_lnfSettings = m_lnfIntegration->configScheme();
+
     auto sourceFile = m_package.fileUrl(QByteArrayLiteral("lockscreen"), QStringLiteral("config.qml"));
     if (sourceFile.isEmpty()) {
         m_ui->lnfConfigWidget->hide();
@@ -280,7 +236,7 @@ KDeclarative::ConfigPropertyMap * ScreenLockerKcm::lnfConfiguration() const
 
 QString ScreenLockerKcm::currentWallpaper() const
 {
-    return m_ui->wallpaperCombo->currentData().toString();
+    return m_ui->kcfg_wallpaperPluginIndex->currentData().toString();
 }
 
 bool ScreenLockerKcm::eventFilter(QObject *watched, QEvent *event)
@@ -291,17 +247,17 @@ bool ScreenLockerKcm::eventFilter(QObject *watched, QEvent *event)
         }
         return false;
     }
-    if (watched != m_ui->wallpaperCombo) {
+    if (watched != m_ui->kcfg_wallpaperPluginIndex) {
         return false;
     }
     if (event->type() == QEvent::Move) {
         if (auto object = m_ui->wallpaperConfigWidget->rootObject()) {
             // QtQuick Layouts have a hardcoded 5 px spacing by default
-            object->setProperty("formAlignment", m_ui->wallpaperCombo->x() + 5);
+            object->setProperty("formAlignment", m_ui->kcfg_wallpaperPluginIndex->x() + 5);
         }
         if (auto object = m_ui->lnfConfigWidget->rootObject()) {
             // QtQuick Layouts have a hardcoded 5 px spacing by default
-            object->setProperty("formAlignment", m_ui->wallpaperCombo->x() + 5);
+            object->setProperty("formAlignment", m_ui->kcfg_wallpaperPluginIndex->x() + 5);
         }
 
     }
