@@ -4,6 +4,7 @@
 
 Copyright (C) 2014 Martin Gräßlin <mgraesslin@kde.org>
 Copyright (C) 2014 Marco Martin <mart@kde.org>
+Copyright (C) 2019 Kevin Ottens <kevin.ottens@enioka.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,7 +23,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <KCModule>
 #include <KPackage/Package>
 
-class KActionCollection;
+class KScreenSaverSettings;
 class ScreenLockerKcmForm;
 
 namespace ScreenLocker
@@ -58,24 +59,25 @@ public Q_SLOTS:
     void load() override;
     void save() override;
     void defaults() override;
-    void test(const QString &plugin);
 
 Q_SIGNALS:
     void wallpaperConfigurationChanged();
     void currentWallpaperChanged();
 
+private Q_SLOTS:
+    void updateState();
+
 private:
-    void shortcutChanged(const QKeySequence &key);
-    bool shouldSaveShortcut();
-    void loadWallpapers();
     void selectWallpaper(const QString &pluginId);
     void loadWallpaperConfig();
     void loadLnfConfig();
     KPackage::Package m_package;
-    KActionCollection *m_actionCollection;
+    KScreenSaverSettings *m_settings;
     ScreenLockerKcmForm *m_ui;
     ScreenLocker::WallpaperIntegration *m_wallpaperIntegration = nullptr;
+    KCoreConfigSkeleton *m_wallpaperSettings = nullptr;
     ScreenLocker::LnFIntegration* m_lnfIntegration = nullptr;
+    KCoreConfigSkeleton *m_lnfSettings = nullptr;
 };
 
 //see https://bugreports.qt.io/browse/QTBUG-57714, don't expose a QWidget as a context property
@@ -91,6 +93,8 @@ public:
         QObject(parent),
         q(parent)
     {
+        connect(q, &ScreenLockerKcm::wallpaperConfigurationChanged, this, &ScreenLockerProxy::wallpaperConfigurationChanged);
+        connect(q, &ScreenLockerKcm::currentWallpaperChanged, this, &ScreenLockerProxy::currentWallpaperChanged);
     }
 
     KDeclarative::ConfigPropertyMap *wallpaperConfiguration() const {
